﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Web

Imports Microsoft.SharePoint
Imports Microsoft.SharePoint.Administration
Imports Microsoft.SharePoint.Administration.Claims
Imports Microsoft.SharePoint.Diagnostics
Imports Microsoft.SharePoint.Utilities

Imports Microsoft.IdentityModel
Imports Microsoft.IdentityModel.Claims
Imports Microsoft.SharePoint.WebControls
Imports System.Data
Imports System.Data.SqlClient
Imports System.Collections
Imports System.Configuration
Imports System.Security.Permissions

Namespace Microsoft.Sample.ContentWeb.ClaimsSecurity
    <Microsoft.SharePoint.Security.SharePointPermission(System.Security.Permissions.SecurityAction.Demand, ObjectModel:=True)> _
 Public Class ContentWebClaimProvider
        Inherits SPClaimProvider
        Private Shared connectionString As String = Nothing


        Public Sub New(ByVal displayName As String)
            MyBase.New(displayName)
            Dim guid As New Guid("56705e15-abd3-44f0-adea-91488da1a572")
            Dim storage As ConnectionStringStorage = Nothing

            ' Read Hierarchical Object Store set by PowerShell scripts.
            If Nothing IsNot SPFarm.Local Then
                storage = CType(SPFarm.Local.GetObject(guid), ConnectionStringStorage)
                If Nothing IsNot storage Then
                    connectionString = storage.ConnectionString
                End If
            End If
        End Sub

        Protected Overrides Sub FillClaimTypes(ByVal claimTypes As List(Of String))
            If Nothing Is claimTypes Then
                Throw New ArgumentNullException("claimTypes")
            End If

            ' Add the claim types that will be added by this claims provider.  
            ' NOTE: For every claim added here please make sure you add a corresponding claimValueTypes.Add line in FillClaimValueTypes method.
            claimTypes.Add(ContentWebClaimTypes.PersonnelNumber)
            claimTypes.Add(ContentWebClaimTypes.Alias)
            claimTypes.Add(ContentWebClaimTypes.FullName)
            claimTypes.Add(ContentWebClaimTypes.CompanyCode)
            claimTypes.Add(ContentWebClaimTypes.UserType)
            claimTypes.Add(ContentWebClaimTypes.UserSubType)
            claimTypes.Add(ContentWebClaimTypes.Manager)
            claimTypes.Add(ContentWebClaimTypes.HROrg)
            claimTypes.Add(ContentWebClaimTypes.HRPro)
            claimTypes.Add(ContentWebClaimTypes.Partner)
            claimTypes.Add(ContentWebClaimTypes.Leader)
            claimTypes.Add(ContentWebClaimTypes.VP)
            claimTypes.Add(ContentWebClaimTypes.CountryCode)
            claimTypes.Add(ContentWebClaimTypes.Level)
            claimTypes.Add(ContentWebClaimTypes.Profession)
            claimTypes.Add(ContentWebClaimTypes.Discipline)
            claimTypes.Add(ContentWebClaimTypes.StandardTitle)
            claimTypes.Add(ContentWebClaimTypes.Blocked)
            claimTypes.Add(ContentWebClaimTypes.BlockedRedirect)
            claimTypes.Add(ContentWebClaimTypes.VPAPersonnelNumber)
            claimTypes.Add(ContentWebClaimTypes.VPAAlias)
            claimTypes.Add(ContentWebClaimTypes.VPAFullName)
            claimTypes.Add(ContentWebClaimTypes.CompoundClaim)
        End Sub

        Protected Overrides Sub FillClaimValueTypes(ByVal claimValueTypes As List(Of String))
            If Nothing Is claimValueTypes Then
                Throw New ArgumentNullException("claimValueTypes")
            End If

            ' Add as many lines as ClaimTypes.Add lines listed in FillClaimTypes method...
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
        End Sub

        ''' <summary>
        ''' Gets the name from a fully qualified name.
        ''' </summary>
        ''' <param name="fullName">The full name. </param>
        ''' <returns>a name string. </returns>
        Private Function GetName(ByVal fullName As String) As String
            Dim aName() As String
            Dim aDelims() As Char = {"/"c, "\"c}

            aName = fullName.Split(aDelims)
            Return aName(aName.GetUpperBound(0))
        End Function


        Protected Overrides Sub FillClaimsForEntity(ByVal context As Uri, ByVal entity As SPClaim, ByVal claims As List(Of SPClaim))
            If Nothing Is entity Then
                Throw New ArgumentNullException("entity")
            End If
            If Nothing Is claims Then
                Throw New ArgumentNullException("claims")
            End If

            Dim userIdClaim As SPClaim = SPClaimProviderManager.DecodeUserIdentifierClaim(entity)
            Dim userAlias As String = GetName(userIdClaim.Value.ToLowerInvariant())

            Try
                Dim userClaimsDataSet As DataSet = Nothing
                Using TempSPMonitoredScope As SPMonitoredScope = New SPMonitoredScope("ContentWeb.CCP.GetUserClaims", 5)
                    ' Retrieve user claims from database, two tables should be returned with the first one for simple claims, the second one for compound claims.
                    userClaimsDataSet = CustomClaimSourceDA.GetUserClaims(userAlias, connectionString)
                End Using

                ' Add simple claims from Tables[0], ignore null values
                If Nothing IsNot userClaimsDataSet AndAlso Nothing IsNot userClaimsDataSet.Tables(0) AndAlso userClaimsDataSet.Tables(0).Rows.Count > 0 Then
                    For Each claimsDataRow As DataRow In userClaimsDataSet.Tables(0).Rows
                        If Not DBNull.Value.Equals(claimsDataRow("PersonnelNumber")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.PersonnelNumber, CStr(claimsDataRow("PersonnelNumber")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                            claims.Add(CreateClaim(ContentWebClaimTypes.Alias, userAlias, Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("FullName")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.FullName, CStr(claimsDataRow("FullName")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("CompanyCode")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.CompanyCode, CStr(claimsDataRow("CompanyCode")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("UserType")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.UserType, CStr(claimsDataRow("UserType")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("UserSubType")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.UserSubType, CStr(claimsDataRow("UserSubType")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Manager")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Manager, CStr(claimsDataRow("Manager")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("HROrg")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.HROrg, CStr(claimsDataRow("HROrg")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("HRPro")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.HRPro, CStr(claimsDataRow("HRPro")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Partner")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Partner, CStr(claimsDataRow("Partner")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Leader")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Leader, CStr(claimsDataRow("Leader")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("VP")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.VP, CStr(claimsDataRow("VP")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("CountryCode")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.CountryCode, CStr(claimsDataRow("CountryCode")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Level")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Level, CStr(claimsDataRow("Level")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Profession")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Profession, CStr(claimsDataRow("Profession")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Discipline")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Discipline, CStr(claimsDataRow("Discipline")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("StandardTitle")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.StandardTitle, CStr(claimsDataRow("StandardTitle")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("Blocked")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.Blocked, CStr(claimsDataRow("Blocked")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("BlockedRedirect")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.BlockedRedirect, CStr(claimsDataRow("BlockedRedirect")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("VPAPersonnelNumber")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAPersonnelNumber, CStr(claimsDataRow("VPAPersonnelNumber")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("VPAAlias")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAAlias, CStr(claimsDataRow("VPAAlias")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                        If Not DBNull.Value.Equals(claimsDataRow("VPAFullName")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAFullName, CStr(claimsDataRow("VPAFullName")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                    Next claimsDataRow
                End If

                ' Add compound claims from Tables[1], ignore null values
                If Nothing IsNot userClaimsDataSet AndAlso Nothing IsNot userClaimsDataSet.Tables(1) AndAlso userClaimsDataSet.Tables(1).Rows.Count > 0 Then
                    For Each claimsDataRow As DataRow In userClaimsDataSet.Tables(1).Rows
                        If Not DBNull.Value.Equals(claimsDataRow("CompoundClaim")) Then
                            claims.Add(CreateClaim(ContentWebClaimTypes.CompoundClaim, CStr(claimsDataRow("CompoundClaim")), Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
                        End If
                    Next claimsDataRow
                End If
            Catch ex As Exception
                claims.Add(CreateClaim(ContentWebClaimTypes.Message, ex.Message & " " & ex.StackTrace, Microsoft.IdentityModel.Claims.ClaimValueTypes.String))
            End Try

        End Sub

        Protected Overrides Sub FillEntityTypes(ByVal entityTypes As List(Of String))
            entityTypes.Add(SPClaimEntityTypes.User)
        End Sub

        ' Used by People Picker in SP UI.
        Protected Overrides Sub FillHierarchy(ByVal context As Uri, ByVal entityTypes() As String, ByVal hierarchyNodeID As String, ByVal numberOfLevels As Integer, ByVal hierarchy As Microsoft.SharePoint.WebControls.SPProviderHierarchyTree)
            If Not EntityTypesContain(entityTypes, SPClaimEntityTypes.User) Then
                Return
            End If

            Select Case hierarchyNodeID
                Case Nothing
                    ' Only add those claims which need to be used for securing/targetting...
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "CompanyCode", "CompanyCode", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "UserType", "UserType", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "UserSubType", "UserSubType", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "Manager", "Manager", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "HROrg", "HROrg", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "HRPro", "HRPro", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "Partner", "Partner", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "Leader", "Leader", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "MSAA", "MSAA", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "VP", "VP", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "CountryCode", "CountryCode", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "Level", "Level", True))
                    hierarchy.AddChild(New SPProviderHierarchyNode(Me.Name, "CompoundClaim", "CompoundClaim", True))

                Case Else
            End Select
        End Sub

        Protected Overrides Sub FillResolve(ByVal context As Uri, ByVal entityTypes() As String, ByVal resolveInput As SPClaim, ByVal resolved As List(Of Microsoft.SharePoint.WebControls.PickerEntity))
            Dim claimTypeName As String = String.Empty
            Dim claimValue As String = String.Empty

            ' Handles only ContentWeb claim types
            If Nothing IsNot resolveInput AndAlso resolveInput.ClaimType.Contains(ContentWebClaimTypes.ContentWebClaimTypePrefix) Then
                claimValue = resolveInput.Value.ToLower()
                claimTypeName = GetName(resolveInput.ClaimType)

                Dim entity As PickerEntity = CreatePickerEntity()
                entity.Claim = resolveInput
                entity.Description = resolveInput.ClaimType & " = " & claimValue
                entity.DisplayText = claimTypeName & " = " & claimValue
                entity.EntityData(PeopleEditorEntityDataKeys.DisplayName) = claimTypeName & " = " & claimValue
                entity.EntityType = SPClaimEntityTypes.User
                entity.IsResolved = True
                resolved.Add(entity)
            End If
        End Sub

        Protected Overrides Sub FillResolve(ByVal context As Uri, ByVal entityTypes() As String, ByVal resolveInput As String, ByVal resolved As List(Of Microsoft.SharePoint.WebControls.PickerEntity))
        End Sub

        Protected Overrides Sub FillSchema(ByVal schema As Microsoft.SharePoint.WebControls.SPProviderSchema)
            Return
        End Sub

        Protected Overrides Sub FillSearch(ByVal context As Uri, ByVal entityTypes() As String, ByVal searchPattern As String, ByVal hierarchyNodeID As String, ByVal maxCount As Integer, ByVal searchTree As Microsoft.SharePoint.WebControls.SPProviderHierarchyTree)
            If Not EntityTypesContain(entityTypes, SPClaimEntityTypes.User) Then
                Return
            End If

            Dim claimValue As String = searchPattern.ToLower()

            Dim claimType As String = ContentWebClaimTypes.ContentWebClaimTypePrefix & "/" & hierarchyNodeID

            If Nothing IsNot hierarchyNodeID AndAlso hierarchyNodeID.Length > 0 Then
                Dim entity As PickerEntity = CreatePickerEntity()
                entity.Claim = CreateClaim(claimType, claimValue, Microsoft.IdentityModel.Claims.ClaimValueTypes.String)
                entity.Description = claimType & " = " & claimValue
                entity.DisplayText = hierarchyNodeID & " = " & claimValue
                entity.EntityData(PeopleEditorEntityDataKeys.DisplayName) = hierarchyNodeID & " = " & claimValue
                entity.EntityType = SPClaimEntityTypes.User
                entity.IsResolved = True
                searchTree.AddEntity(entity)
            End If
        End Sub

        Friend Shared ReadOnly Property ProviderDisplayName() As String
            Get
                Return "ContentWeb Claim Provider"
            End Get
        End Property

        Friend Shared ReadOnly Property ProviderInternalName() As String
            Get
                Return "ContentWebClaimProvider"
            End Get
        End Property

        Public Overrides ReadOnly Property Name() As String
            Get
                Return ProviderInternalName
            End Get
        End Property

        Public Overrides ReadOnly Property SupportsEntityInformation() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property SupportsHierarchy() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property SupportsResolve() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property SupportsSearch() As Boolean
            Get
                Return True
            End Get
        End Property
    End Class


End Namespace